@@ -4,6 +4,7 @@ import sys
 import os
 import platform
 import json
+import shutil
 from os.path import abspath, expanduser, normpath, exists, basename
 
 CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])
@@ -31,6 +32,17 @@ def destroyTemplate(dir):
             os.unlink(path)
     os.rmdir(dir)
 
+def copytree(src, dst, symlinks=False, ignore=None, exclude=['.git']):
+    for item in os.listdir(src):
+        if item in exclude:
+            continue
+        s = os.path.join(src, item)
+        d = os.path.join(dst, item)
+        if os.path.isdir(s):
+            shutil.copytree(s, d, symlinks, ignore)
+        else:
+            shutil.copy2(s, d)
+
 LN=choose('mklink /D', 'ln -s')
 
 
@@ -78,11 +90,60 @@ def main():
         config = json.load(fconf)
 
 
+@main.command()
+@click.argument('alias')
+def update(alias):
+    """
+    Actualiza el template identificado con el ALIAS.
+    """
+    print(f"cd {texpath()}/{alias}")
+    print("git pull")
+    os.system(f"cd {texpath()}/{alias}; git pull")
+
+
+def wdescription(path, description):
+    with open(f"{path}/.description.texpj", 'w') as f:
+        f.write(description)
+
+def rdescription(path):
+    content = ''
+
+    if exists(f"{path}/.description.texpj"):
+        with open(f"{path}/.description.texpj", 'r') as f:
+            content = f.read()
+
+    if len(content.strip()) == 0:
+        content = 'sin descripción'
+    return content
+
+
+@main.command()
+@click.argument('directory')
+@click.argument('alias')
+@click.argument('description', required=False)
+def add(directory, alias, description):
+    """
+    Registra DIRECTORY como una plantilla de latex y lo identifica con ALIAS.
+
+    \b
+    $ texpj add . elec_doc
+    $ texpj add reporte report
+    """
+    
+    if exists(f"{texpath()}/{alias}"):
+        print("ERROR: los alias deben únicos")
+        return
+
+    print(f"{LN} {abspath(expanduser(directory))} {texpath()}/{alias}")
+    os.system(f"{LN} {abspath(expanduser(directory))} {texpath()}/{alias}")
+    wdescription(f"{texpath()}/{alias}", '' if description == None else description)
+
 @main.command()
 @click.argument('template')
 @click.argument('alias')
+@click.argument('description', required=False)
 @click.option('--url/--no-url', default=False, help="indica si es un url")
-def install(template, alias, url):
+def install(template, alias, description, url):
     """
     Instala el TEMPLATE de forma local y lo regitra como ALIAS.
 
@@ -95,32 +156,19 @@ def install(template, alias, url):
     $ texpj install BenyaminGaleano/report
     $ texpj install https://github.com/BenyaminGaleano/report.git --url
     """
-    print(f'Falta implementación, {template} {alias} url:{url}')
 
+    if exists(f"{texpath()}/{alias}"):
+        print("ERROR: los alias deben únicos")
+        return
 
-@main.command()
-@click.argument('alias')
-def update(alias):
-    """
-    Actualiza el template identificado con el ALIAS.
-    """
-    print(f'Falta implementación, {alias}')
-
-
-@main.command()
-@click.argument('directory')
-@click.argument('alias')
-def add(directory, alias):
-    """
-    Registra DIRECTORY como una plantilla de latex y lo identifica con ALIAS.
-
-    \b
-    $ texpj add . elec_doc
-    $ texpj add reporte report
-    """
-    print(f"{LN} {abspath(expanduser(directory))} {texpath()}/{alias}")
-    os.system(f"{LN} {abspath(expanduser(directory))} {texpath()}/{alias}")
+    if url:
+        print(f"git clone {template} {texpath()}/{alias}")
+        os.system(f"git clone {template} {texpath()}/{alias}")
+    else:
+        print(f"git clone https://github.com/{template}.git {texpath()}/{alias}")
+        os.system(f"git clone https://github.com/{template}.git {texpath()}/{alias}")
 
+    wdescription(f"{texpath()}/{alias}", '' if description == None else description)
 
 @main.command()
 @click.argument('alias')
@@ -136,16 +184,28 @@ def remove(alias):
 
 @main.command()
 def list():
+    """
+    Lista todas las posibles plantillas guardadas.
+    """
     print("Plantillas en el directorio:")
     for template in os.listdir(texpath()):
         if not os.path.isdir(f"{texpath()}/{template}"):
             continue
-        print(f'\t{template}')
+        print(f' {template:20}', rdescription(f"{texpath()}/{template}"))
+
+@main.command()
+@click.argument('alias')
+@click.argument('description')
+def describe(alias, description):
+    """
+    Cambia la descripción de ALIAS por DESCRIPTION.
+    """
+    wdescription(f"{texpath()}/{alias}", description)
 
 @main.command()
 @click.argument('alias')
 @click.argument('name')
-@click.option('--directory', '-d', help="directorio base")
+@click.option('--directory', '-d', help="directorio base", default="")
 def create(alias, name, directory):
     """
     Crea una copia del template registrado como ALIAS en la posición actual o en 
@@ -155,7 +215,20 @@ def create(alias, name, directory):
     $ texpj create report lab1
     $ texpj create report lab1 -d ~/documentos/latex
     """
-    print(f'Falta implementación, {alias}, {name}, {directory}')
+    
+    print(f"copiando {texpath()}/{alias} a {abspath(expanduser(os.path.join(directory, name)))}")
+    copytree(f"{texpath()}/{alias}", f"{abspath(expanduser(os.path.join(directory, name)))}")
+
+
+@main.command()
+@click.argument('files', nargs=-1)
+def launch(files):
+    """
+    Se encarga de abrir los archivos.
+    """
+    for f in files:
+        print(f'Abriendo {f}')
+        os.popen(f"open {abspath(expanduser(f))}")
 
 if __name__ == "__main__":
     main()
